# -*- coding: utf-8 -*-
__author__ = 'renato'


import jsonpickle
from li_common.helpers import to_camel_case

def serialize(cls, data_type=True):
    if data_type:
        return jsonpickle.encode(cls)
    else:
        json_str = jsonpickle.encode(cls, unpicklable=False)
        json_dict = jsonpickle.decode(json_str)
        data = loop_dic(json_dict)
        return jsonpickle.encode(data, unpicklable=False)


def descerialize(values, cls=None):
    obj = jsonpickle.decode(values)
    result = None

    if cls is not None and type(obj) is dict:
        result = cls.from_dic(obj)
    else:
        result = obj

    return result

def loop_dic(values):
    data = {}
    for key, value in values.items():
        # print key, value
        if type(value) is dict:
            data[to_camel_case(key)] = loop_dic(value)
        elif type(value) is list:
            data[to_camel_case(key)] = loop_list(value)
        else:
            data[to_camel_case(key)] = value

    return data


def loop_list(values):
    data = []
    for value in values:
        if type(value) is dict:
            data.append(loop_dic(value))
        elif type(value) is list:
            data.append(loop_list(value))

    return data


# -*- coding: utf-8 -*-
import os

from celery import Celery

class WorkerConnect(object):

    @classmethod
    def __init__(self, broker_url=None, backend_url=None):

        if broker_url is None:
            broker_url = os.environ.get('WORKER_BROKER_URL')

        if backend_url is None:
            backend_url = os.environ.get('WORKER_BACKEND_URL')

        if hasattr(self,'celery') is False:
            self.celery = Celery('WorkerConnect', broker=broker_url, backend=backend_url)
        else:
            pass

    def execute(self, task_name, option='delay', countdown=0, args=None):

        if args is not None:
            data = args
        else:
            data = None

        if not data:
            return { 'status': 'erro', 'code': 500, 'msg': 'no args' }

        if option == 'get':
            celery_response = self.celery.signature(task_name, kwargs=data, countdown=countdown).delay().get(timeout=60)
            return { 'status': 'success', 'code': 200, 'data': celery_response }
        else:
            celery_response = self.celery.signature(task_name, kwargs=data, countdown=countdown).delay()

            if celery_response.status == 'SUCCESS' or celery_response.status == 'PENDING':
                return { 'status': 'success', 'code': 200 }
            else:
                return { 'status': 'success', 'code': 500 }


# -*- coding: utf-8 -*-

"""
Funções para auxiliarem no desenvolvimento
"""

import os
import time
import re
from unicodedata import normalize
from functools import wraps

import logging
LOGGER = logging.getLogger()


def tente_outra_vez(excecoes, tentativas=4, tempo_espera=3, multiplicador_espera=1):
    """
    Escuta pelas exceções especificadas e refaz a chamada para o método.
    :param excecoes: As exceções que devem ser escutadas
    :type excecoes: Exception or tuple
    :param tentativas: Número de vezes que deve ser chamado o método
    :type tentativas: int
    :param tempo_espera: Tempo inicial de espera em segundos entre as tentativas
    :type tempo_espera: int
    :param multiplicador_espera: Multiplicador para o tempo de espera nas chamadas subsequentes.
    :type multiplicador_espera: int
    """
    def deco_tente_outra_vez(funcao):
        """
        Decorator para tente_outra_vez
        """
        @wraps(funcao)
        def funcao_tente_outra_vez(*args, **kwargs):
            """
            Tratamento de decorator para tente_outra_vez
            """
            _tentativas, _tempo_espera = tentativas, tempo_espera
            while _tentativas > 1:
                try:
                    return funcao(*args, **kwargs)
                except excecoes, exc:
                    msg = "{}, Reconectando in {} segundos...".format(str(exc), _tempo_espera)
                    if LOGGER:
                        LOGGER.warning(msg)
                    time.sleep(_tempo_espera)
                    _tentativas -= 1
                    _tempo_espera *= multiplicador_espera
            return funcao(*args, **kwargs)
        return funcao_tente_outra_vez
    return deco_tente_outra_vez


def remover_acentos(value):
    """Normalize the values."""
    try:
        return normalize('NFKD', value.decode('utf-8')).encode('ASCII', 'ignore')
    except UnicodeEncodeError:
        return normalize('NFKD', value).encode('ASCII', 'ignore')


def pegar_versao_git():
    """
    pegar_versao_git
    :return: git version
    """
    import subprocess

    try:
        version = subprocess.check_output(['git', 'rev-parse', '--short', 'HEAD'])
        version = version.replace("\n","")
        return version
    except subprocess.CalledProcessError:
        if os.path.isfile('VERSION') is True:
            with open('VERSION','r') as f:
                version = f.read()
            version = version.replace("\n","")
            return version
        else:
            return '0'


def send_email(template_file=None, context=None,
               contrato_id=None, conta_id=None, usuario_id=None, cliente_id=None, pedido_venda_id=None,
               salva_evidencia=True, to_email=None, reply_to=None, countdown=0):
    """
    send_email
    :param template_file: template_file
    :param context: context
    :param contrato_id: contrato_id
    :param conta_id: conta_id
    :param usuario_id: usuario_id
    :param cliente_id: cliente_id
    :param pedido_venda_id: pedido_venda_id
    :param salva_evidencia: salva_evidencia
    :param to_email: to_email
    :return: None
    """
    from li_common.conexoes.worker import WorkerConnect

    args = {
        "template_file": template_file,
        "context": context,
        "contrato_id": contrato_id,
        "conta_id": conta_id,
        "usuario_id": usuario_id,
        "cliente_id": cliente_id,
        "pedido_venda_id": pedido_venda_id,
        "salva_evidencia": salva_evidencia,
        "to_email": to_email,
        "reply_to": reply_to
    }

    WorkerConnect().execute('async.utils.send_email_template', args=args, countdown=countdown)


def send_email_sem_template(from_email=None, to_email=None, reply_to=None, subject=None,
            html_message=None, text_message=None, countdown=0, salva_evidencia=True
):
    """
    send_email_sem_template
    :param from_email: from_email
    :param to_email: to_email
    :param reply_to: reply_to
    :param subject: subject
    :param content: content
    :param countdown: countdown
    :return: None
    """
    from li_common.conexoes.worker import WorkerConnect

    args = {
        "from_email": from_email,
        "to_email": to_email,
        "reply_to": reply_to,
        "subject": subject,
        "html_message": html_message,
        "text_message": text_message,
        "salva_evidencia": salva_evidencia
    }

    WorkerConnect().execute('async.utils.send_email', args=args, countdown=countdown)


def carregar_env(caminho_arquivo=None):
    if not caminho_arquivo:
        caminho_arquivo = '/etc/profile.d/env.sh'
    if not os.path.exists(caminho_arquivo):
        return 'ARQUIVO NAO EXISTE'
    try:
        arquivo = open(caminho_arquivo)
        for linha in arquivo:
            linha = linha.replace('\n', '').replace('export ', '')
            if not linha or '=' not in linha:
                continue
            chave, valor = linha.split('=')
            try:
                os.environ[chave.strip()] = valor.strip().strip('"').strip("'")
            except Exception, e:
                print '{} - ERRO: {}'.format(linha, str(e))
        arquivo.close()
    except Exception, e:
        print str(e)


def to_camel_case(snake_str):
    components = snake_str.split('_')
    return components[0] + "".join(x.title() for x in components[1:])


def to_underscore_case(name):
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()


# -*- coding: utf-8 -*-

"""
Componentes para permitir a criação de objetos de extensões.
"""
from importlib import import_module

SETTINGS = None


class ClasseNaoEncontrada(Exception):
    """
    Erro disparado quando uma classe não é encontrada no módulo entidades.
    """
    pass


class ClasseInvalida(Exception):
    """
    Disparado quando a instância da classe não pode ser criada por algum TypeError.
    """
    pass


class ExtensaoNaoDefinido(Exception):
    """
    Disparado quando a extensão não for definida no SETTINGS.
    """
    pass


class ModuloDeExtensaoNaoEncontrado(Exception):
    """
    Disparado quando der ImportError do módulo definido no SETTINGS.
    """
    pass


class Extensao(object):
    """
    Fornece a capacidade de uma classe poder criar extensões baseados em um modulo definido como o atributo da classe 'extensao'.
    """
    extensao = None

    def _cria_extensao(self, tipo, modulo, **kwargs):
        """
        Método que executa a ação de criar a extensão. É usado pelos dois métodos públicos de criar extensão.
        :param tipo: O nome da classe que deverá ser encontrada no módulo <extensao>.<modulo>
        :type tipo: str
        :param modulo: O módulo de onde a classe deve ser instanciada.
        :type modulo: str
        :param kwargs: Argumentos a serem passados para a criação da instância da classe
        :return: Uma instância da classe criada.
        """
        try:
            import_extensao = SETTINGS.EXTENSOES[self.extensao]
        except KeyError:
            raise ExtensaoNaoDefinido(u'A extensão "{}" não foi encontrada nas configurações. Você esqueceu de inclui-la em SETTINGS.EXTENSOES?'.format(self.extensao))
        try:
            _modulo = import_module('{}.{}'.format(import_extensao, modulo))
        except ImportError:
            raise ModuloDeExtensaoNaoEncontrado(u'Não foi encontrado o modulo "{}.{}" para a extensão "{}" no ambiente.'.format(import_extensao, modulo, self.extensao))
        try:
            classe = getattr(_modulo, tipo)
        except AttributeError:
            raise ClasseNaoEncontrada(u'Não foi encontrado a classe "{}.{}.{}" para a extensão "{}" no ambiente.'.format(import_extensao, modulo, tipo, self.extensao))
        try:
            instancia = classe(**kwargs)
            if hasattr(instancia, 'extensao'):
                instancia.extensao = self.extensao
            return instancia
        except TypeError:
            raise ClasseInvalida(u'A classe "{}.{}.{}" para o meio de pagamento "{}" não aceitou os argumentos passados: "{}".'.format(import_extensao, modulo, tipo, self.extensao, kwargs))

    def cria_entidade_extensao(self, tipo, **kwargs):
        """
        Retorna a entidade definida em tipo contendo dentro do módulo 'entidades' da extensão.
        :param tipo: O nome da classe que deverá ser encontrada no módulo <extensao>.entidades
        :type tipo: str
        :param kwargs: Argumentos a serem passados para a criação da instância da classe
        :return: Uma instância da classe criada.
        """
        return self._cria_extensao(tipo, 'entidades', **kwargs)

    def cria_servico_extensao(self, tipo, **kwargs):
        """
        Retorna o serviço definido em tipo contendo dentro do módulo 'servicos' da extensão.
        :param tipo: O nome da classe que deverá ser encontrada no módulo <extensao>.servicos
        :type tipo: str
        :param kwargs: Argumentos a serem passados para a criação da instância da classe
        :return: Uma instância da classe criada.
        """
        return self._cria_extensao(tipo, 'servicos', **kwargs)



# -*- coding: utf-8 -*-

"""
Módulo contendo a base para todos os serviços usados nos projetos da Loja Integrada.
"""
from importlib import import_module
from li_common.comunicacao import requisicao
from li_common.padroes import extensibilidade, serializacao


class EntidadeNaoEncontrada(Exception):
    """
    Disparada quando a entidade passada não for encontrada no modulo.
    """
    pass


class ParamentrosDeEntidadeInvalidos(Exception):
    """
    Disparada quando o init da classe não aceita os parâmetros passados.
    """
    pass


class Servico(extensibilidade.Extensao):
    """
    Classe base para os serviços. Já define a extensibilidade do serviço.
    """
    formatador = serializacao.Formatador

    @classmethod
    def cria_entidade(cls, nome_modulo, entidade, **kwargs):
        """
        Factory para instancias de entidades. Use esse método para evitar import circular entre os módulos de servicos e entidades
        :param nome_modulo: Nome completo do módulo onde está a entidade. Ex.: pagador.entidades
        :type nome_modulo: str
        :param entidade: Nome da classe da entidade.
        :type entidade: str
        :param kwargs: Argumentos que devem ser passados para o init da classe.
        :type kwargs: dict
        :return: A instância da classe.
        """
        modulo = import_module('{}.entidades'.format(nome_modulo))
        try:
            classe = getattr(modulo, entidade)
        except AttributeError, ex:
            raise EntidadeNaoEncontrada(u'Não foi encontrado a classe "{}.entidades.{}" no ambiente. {}'.format(nome_modulo, entidade, ex))
        try:
            return classe(**kwargs)
        except TypeError:
            raise ParamentrosDeEntidadeInvalidos(u'A classe "{}.entidades.{}" não aceitou os argumentos passados: "{}".'.format(nome_modulo, entidade, kwargs))

    @classmethod
    def obter_conexao(cls, formato_envio=requisicao.Formato.json, formato_resposta=requisicao.Formato.json, headers=None):
        """
        Factory para criação de objeto conexão para realizar chamadas HTTP.
        :param formato_envio: Qual o formato que os dados deverão ser passados nas chamadas HTTP da conexão.
        :type formato_envio: li_common.comunicacao.requisicao.Formato
        :param formato_resposta: Qual o formato que os dados serão respondido nas chamadas HTTP da conexão.
        :type formato_resposta: li_common.comunicacao.requisicao.Formato
        :param headers: O cabeçalho HTTP a ser enviado na chamada.
        :type headers: dict
        :return: Uma instância de li_common.comunicacao.requisicao.Conexao
        :rtype: li_common.comunicacao.requisicao.Conexao
        """
        return requisicao.Conexao(formato_envio=formato_envio, formato_resposta=formato_resposta, headers=headers)



# -*- coding: utf-8 -*-
"""
Fornece recusros para a montagem de formulário para as APIs
"""
from li_common.padroes import entidades, serializacao


class TipoDeCampo(object):
    """
    Traduções de nomes de campo para o renderizador de campos
    """
    texto = "CharField"
    area = "TextArea"
    boleano = "BooleanField"
    escolha = "ChoiceField"
    decimal = "BRDecimalField"
    oculto = "Hidden"


class FormatoDeCampo(object):
    """
    Tipos de formato de campo para que o valor do campo seja formatado ao ser renderizado
    """
    texto = "texto"
    ascii = "ascii"
    html = "html"
    xml = "xml"
    json = "json"
    numero = "numero"


class CampoFormulario(entidades.Entidade):
    """
    Define as informações necessárias para construir um campo de formulário no renderizador
    """
    _nao_serializar = ['validador']

    def __init__(self, nome, label=None, bound=True, descricao=None, requerido=False, tipo=TipoDeCampo.texto, formato=FormatoDeCampo.texto, invisivel=False,
                 tamanho_min=None, tamanho_max=None, ordem=0, texto_ajuda=None, opcoes=(), grupo=None, valor_padrao=None, validador=None, decimais=2):
        self.bound = bound
        self.ordem = ordem
        self.nome = nome
        self.label = label
        self.descricao = descricao
        self.requerido = requerido
        self.tipo = tipo
        self.formato = formato
        self.invisivel = invisivel
        self.tamanho_min = tamanho_min
        self.tamanho_max = tamanho_max
        self.texto_ajuda = texto_ajuda
        self.opcoes = opcoes
        self.grupo = grupo
        self.valor_padrao = valor_padrao
        self.validador = validador
        self.decimais = decimais


class DadosInvalidos(Exception):
    """
    Exceção para quando ocorrer erros de validação. Ela possui atributo erros, que é um dicionário contendo o erro de cada validação.
    """
    def __init__(self, msg, erros):
        super(DadosInvalidos, self).__init__(msg)
        self.erros = erros


class ValidadorBase(object):
    """
    Base para processar validações
    """
    def __init__(self, valor, valores):
        self.valores = valores
        self.valor = valor
        self.erros = {}

    @property
    def eh_valido(self):
        """
        É chamado pelo processo de validação do formulário. Deve ser sobrescrito com a regra de validação
        :return True caso o valor esteja correto
        :rtype: bool
        """
        return True


class Formulario(entidades.Entidade):
    """
    Classe base para a criação de um formulário para cadastro de entidades.
    """
    def __init__(self):
        self._campos = [atributo for atributo in dir(self) if atributo != 'campos' and atributo != 'nomes_campos' and isinstance(getattr(self, atributo), CampoFormulario)]

    @property
    def campos(self):
        """
        Um dicionário com os campos do formulário, tendo o nome do atributo como chave e a instância de CampoFormulario como valor
        :return: A lista de campos
        :rtype: dict
        """
        return {campo: getattr(self, campo) for campo in self._campos}

    @property
    def nomes_campos(self):
        """
        Uma lista contendo os nomes dos campos definidos no atributo CampoFormulario.nome
        :return: A lista dos nomes dos campos
        :rtype: list
        """
        return [getattr(self, campo).nome for campo in self._campos]

    def to_dict(self):
        """
        Renderiza os campos do formulário em dicionário
        :return: Um dicionario com os campos do formulário em dicionário
        :rtype: dict
        """
        return {campo: self.campos[campo].to_dict() for campo in self.campos}

    def validar_valores(self, valores):
        """
        Valida se os valores passados para os campos estão de acordo com a regra de validação caso seja definida no campo
        :param valores: Um dicionário contendo os valores a serem validados sendo a chave o nome atributo do campo
        :type valores: dict
        :return: Um dicionário com os erros encontrados ou um dicionário vazio.
        :rtype: dict
        """
        erros = {}
        for campo in self.campos:
            campo = self.campos[campo]
            if campo.validador:
                valor = valores.get(campo.nome, None)
                validacao = campo.validador(valor, valores)
                if not validacao.eh_valido:
                    erros[campo.nome] = validacao.erros
        if erros:
            raise DadosInvalidos(u"Ocorreram erros de validação.", erros)

    def definir_valores(self, instancia, valores):
        """
        Define os valores dos campos do formulário em um instancia, formatando os valores de acordo com o campo.formato
        :param instancia: Uma instância de um objeto.
        :type instancia: object
        :param valores: Dicionário contendo os valores. As chaves deve ser os mesmos que campo.nome
        :type valores: dict
        :return: None
        """
        for campo in self.campos:
            campo = self.campos[campo]
            if campo.bound:
                valor = valores.get(campo.nome, None)
                if valor and campo.formato == FormatoDeCampo.ascii:
                    valor = serializacao.Formatador.string_para_ascii(valor)
                setattr(instancia, campo.nome, valor)



# -*- coding: utf-8 -*-
from django.db import connection


class Query(object):
    def __init__(self, sql):
        self.sql = sql
        self._resultados = []

    def __getitem__(self, item):
        return self.resultado[item]

    def __nonzero__(self):
        return len(self._resultados) > 0

    @property
    def resultado(self):
        if self.tem_resultados:
            return self._resultados[0]
        return {}

    @property
    def resultados(self):
        if self.tem_resultados:
            return self._resultados
        return []

    def executar(self, *args):
        cursor = connection.cursor()
        cursor.execute(self.sql, args)
        desc = cursor.description

        if cursor.statusmessage.startswith("INSERT") \
                or cursor.statusmessage.startswith("UPDATE") \
                or cursor.statusmessage.startswith("DELETE"):
            return self

        self._resultados = [
            dict(zip([col[0] for col in desc], row))
            for row in cursor.fetchall()
        ]

        return self

    @property
    def tem_resultados(self):
        return len(self._resultados) > 0


# class QueryMock(Query):
#     """
#     Serve para fazer um mock simples do resultado do Query
#     """
#
#     def executar(self, *args):
#         r = self.mock()
#         if isinstance(r, list):
#             self._resultados = r
#         else:
#             self._resultados = [r]
#         return self
#
#     def mock(self):
#         """
#         Responsável por retornar o resultado que será usado no mock
#         """
#         return {}